import { useCallback, useEffect, useState } from "react"

export enum Key {
  LAST_VAULT_PATH = "app.state.last_vault_path",
  PREFERRED_LOCALE = "app.config.locale",
  ENABLE_AUTO_LOCK = "app.config.enable_auto_lock",
  AUTO_LOCK_AFTER = "app.config.auto_lock_after",
}

interface StoredData {
  [Key.LAST_VAULT_PATH]: string
  [Key.PREFERRED_LOCALE]: string
  [Key.ENABLE_AUTO_LOCK]: boolean
  [Key.AUTO_LOCK_AFTER]: number
}

const events = new Map(Object.values(Key).map(key => [key, new Set()])) as {
  get<K extends Key>(key: K): Set<(value: StoredData[Key]) => void>
}

export function useStorage<K extends Key>(key: K) {
  const [state, setState] = useState(get(key)!)
  useEffect(() => {
    events.get(key).add(setState as any)
    return () => {
      events.get(key).delete(setState as any)
    }
  }, [key])
  const setState2 = useCallback(
    (value: StoredData[K]) => {
      set(key, value)
    },
    [key]
  )

  return [state, setState2] as const
}

export function get<K extends Key>(key: K): StoredData[K] | undefined {
  try {
    const value = localStorage.getItem(key)
    return value == null ? undefined : (JSON.parse(value!) as StoredData[K])
  } catch {}
}

export function set<K extends Key>(key: K, value: StoredData[K]) {
  try {
    localStorage.setItem(key, JSON.stringify(value))
    events.get(key).forEach(fn => fn(value))
  } catch (e) {
    console.error(e)
  }
}

export function remove(key: Key) {
  try {
    localStorage.removeItem(key)
  } catch {}
}

const defaults: typeof set = (key, value) => {
  if (!(key in localStorage)) {
    set(key, value)
  }
}
defaults(Key.ENABLE_AUTO_LOCK, true)
defaults(Key.AUTO_LOCK_AFTER, 120)
